/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
if(!dojo._hasResource["zero.feed.Feed"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["zero.feed.Feed"] = true;
dojo.provide("zero.feed.Feed");

zero.feed.FeedFactory = {
	_toList : function(argList){
		// avoid getting array of chars	
		if ( typeof argList == "string") {
			return [argList];
		}
		var list = [];
		for (var i = 0; i < argList.length; i++) {
			var arg = argList[i];
			if (arg instanceof Array) {
				for (var j = 0; j < arg.length; j++) {
					list.push(arg[j]);
				}
			} else {
				list.push(arg);
			}
		}
		return list;
	},
	
	fetch : function(/*string*/ url){
		// summary:
		// 	fetch remote feed over URL.
		var urls = [url];
		return new zero.feed.Feed(urls);
	},
	
	aggregate : function(){
		// summary:
		//	aggregate multiple feeds over array of URLs.
		var urls = this._toList(arguments);
		return new zero.feed.Feed(urls);
	}
};

dojo.declare("zero.feed.Feed", null,{
    // summary:
	// 	Provide common manipulation functions for deminishing feed.
	// description:
	//  All the functions can be chaining.
	
	constructor: function(urls){
		this.urls = urls;
		this.operators = [];
	},
	
	urls: null,
    operators: null,
    namespaces : null,
	/*
	 * @param {Array} nsMap     //The namespaces map of <prefix, uri>
	 */
	declareNamespaces: function(nsMap){
		this.namespaces = nsMap;
		return this;
	},
	
	/*
	 * @param {Map} query     // projecting or grouping feed entries.
	 */
	query: function(queryMap){
		this.operators.push({
			"operator":"query", 
			"params": queryMap
		});
		return this;
	},
	
	filter : function() {
		if(arguments.length > 1) {
			var err = new Error("Only accept 1 argument in function filter()!");
			// err.name = "";
			throw err;
		}
		return this._abstractFilter("filter", arguments[0]);
	},
	
	exclude : function() {
		if(arguments.length > 1) throw new Error("Only accept 1 argument in function exclude()!");
		return this._abstractFilter("exclude", arguments[0]);
	},
	
	/*
	 * @param {Array} keywords     //Filter entry list by keywords
	 */
	filterByKeywords: function(){
		return this._abstractFilter("filter", {keywords: arguments});
	},
	
	/*
	 * @param {xpath} XPath expression    //Filter entry list by XPath
	 */
	filterByXPath: function(xpath){
		return this._abstractFilter("filter", {XPath : xpath});
	},
	
	/*
	 * @param {datetimeOrDuration} Date time or duration
	 */
	before: function(datetimeOrDuration) {
		return this.filter({before:datetimeOrDuration});
	},
	
	/*
	 * @param {datetimeOrDuration} Date time or duration
	 */
	after: function(datetimeOrDuration) {
		return this.filter({after:datetimeOrDuration});
	},
	
	/*
	 * @param {int} count       //Truncate entry list by given count
	 */
	top: function(count){
		if(count < 1 || isNaN(count)){
			throw new Error("illegal parameter: " + count);
		}
		this.operators.push({
			"operator":"top", 
			"count": count
		});
		return this;
	},
	
	/*
	 * @param {string} elementName     //Remove the duplicated entries with same element value
	 */
	uniqueBy: function(elementName){
		this.operators.push({
			"operator":"uniqueBy", 
			"by": elementName
		});
		return this;
	},
	/*
	 * Diminish feed with unique entries by entry link. 
	 */
	uniqueByLink: function() {
		return this.uniqueBy("link");
	},
	
	/*
	 * Diminish feed with unique entries by entry id. 
	 */
	uniqueByID: function() {
		return this.uniqueBy("id");
	},
	
	/*
	 * @param {string} element     //Sort the entry list by element value
	 */
	orderBy: function(elementName, ascend){
		this.operators.push({
			"operator":"orderBy", 
			"by": elementName,
			"ascend": ascend !== true
		});
		return this;
	},
	
	/*
	 * @param {Function} load     //Callback function for load the result successfully
	 */
	loadAsXML: function(load, error){
		this.load("xml", load, error);
	},
	
	/*
	 * @param {Function} load     //Callback function for load the result successfully
	 */
	loadAsJSON: function(load, error){
		this.load("json", load, error);
	},
	
	/*
	 * Synchronous invocation.
	 * @param {string} format you can use "xml" or "json", default value is "xml".
	 */
	getResult: function(format) {
		var self = this;
		return this.load(
				(format) ? format : "xml", 
				function() {}, 
				self._errorHandler, 
				false	// sync
			   );
	},
	
	/*
	 * @param {string} filterName 	// "filter" or "exclude"
	 * @param {object} arg 			// argument in type of object
	 */
	_abstractFilter : function (/* string */ filterName, arg) {
		// arguments check
		for(var i in arg){
			if(i != "keywords" && i != "XPath" && i != "before" && i != "after"){
				throw new Error("illegal member name in argument: " + i);
			}
		}
		if(arg["keywords"]) {
			var keywords = zero.feed.FeedFactory._toList(arg.keywords);
			this.operators.push({
				"operator":filterName, 
				"keywords": keywords
			});
		} 
		if (arg["XPath"]) {
			this.operators.push({
				"operator":filterName, 
				"xpath": arg.XPath
			});
		} 
		if (arg["before"]) {
			var datetimeOrDuration = arg.before;
			this.operators.push({
				"operator":filterName, 
				"xpath": "zero:before(atom:updated, '" + datetimeOrDuration + "')"
			});
		}
		if (arg["after"]) {
			var datetimeOrDuration = arg.after;
			this.operators.push({
				"operator":filterName, 
				"xpath": "zero:after(atom:updated, '" + datetimeOrDuration + "')"
			});
		}
		return this;
	},
	
	load: function(format, callback, error, async){
		
		var content =  dojo.toJson({
				"urls": this.urls, 
				"operators": this.operators, 
				"namespaces": this.namespaces,
				"format": format
			});
		var self = this;
		// default in xml
		var handleAS = (format == "json")? "json" : "xml";
		var syncResult;
		
		dojo.rawXhrPost({
		    url: dojo.moduleUrl("zero", "../assemble/feedAPI"),  
		    postData: content,
		    contentType: "application/json",
		    handleAs: handleAS,
		    handle: function(data, ioArgs) {
				callback(data, ioArgs);
				syncResult = data;
		    },
		    sync: async == false,
		    error: (error) ? error : self._errorHandler
		});
		if(async == false) {
			return syncResult;
		}
		
	},
	/*
	 * default error handler
	 */
	_errorHandler: function(error, args) {
		
		if(error.responseText != null || error.responseText != ""){
			var exception = dojo.fromJson(error.responseText);
			delete error.responseText;
			error["rootCause"] = exception["rootCause"];
		}
		throw error;
	}
});

}